package uk.me.piggott.simplechat;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Vector;
	
public class ServerImpl implements Server {

	private Vector<Client> clients = new Vector<Client>();
	private ArrayList<String> messages = new ArrayList<String>();
	private SimpleDateFormat df;
	private Registry registry;
	private GUI gui;
	
    public ServerImpl(GUI gui) throws RemoteException {
    	registry = LocateRegistry.createRegistry(Registry.REGISTRY_PORT);
	    registry.rebind("SimpleChat", (Server) UnicastRemoteObject.exportObject(this, 0));
		df = new SimpleDateFormat("HH:mm:ss z");
		
		this.gui = gui;
		
		messages.add("("+df.format(new Date())+") SimpleChatServer started");
		gui.showMessage("("+df.format(new Date())+") SimpleChatServer started");
    }
    
    public void sendMessage(String message) {
    	messages.add("("+df.format(new Date())+") "+message);
    	gui.showMessage("("+df.format(new Date())+") "+message);
    	notifyClients();
	}

    private void notifyClients() { 
    	Vector<Client> deadSubs = null; 
    	for (Client client: clients) { 
    		try { client.update(); } 
    		catch (java.rmi.ConnectException ce) { 
    			if (deadSubs == null) deadSubs = new Vector<Client>(); 
    			deadSubs.addElement(client); 
    		} 
    		catch (java.rmi.NoSuchObjectException nsoe){ 
    			if (deadSubs == null) deadSubs = new Vector<Client>(); 
    			deadSubs.addElement(client);
    		} 
    		catch (java.rmi.RemoteException re) { 
    		} 
    	} 
    	if (deadSubs != null) { 
    		for (Client client: deadSubs) {
    			clients.remove(client);
    		}
    	} 
    } 

    public void subscribe(Client client) {
    	clients.add(client);
    	try {
			messages.add("("+df.format(new Date())+") "+client.getName()+" connected");
			gui.showMessage("("+df.format(new Date())+") "+client.getName()+" connected");
		} catch (RemoteException e) {
			messages.add("("+df.format(new Date())+") A user attempted to connect and failed.");
			gui.showMessage("("+df.format(new Date())+") A user attempted to connect and failed.");
		}
    	
    	notifyClients();
	}
    
    public void unSubscribe(Client client) {
    	clients.remove(client);
    	try {
			messages.add("("+df.format(new Date())+") "+client.getName()+" disconnected");
			gui.showMessage("("+df.format(new Date())+") "+client.getName()+" disconnected");
		} catch (RemoteException e) {
			messages.add("("+df.format(new Date())+") A user disconnected.");
			gui.showMessage("("+df.format(new Date())+") A user disconnected.");
		}
    	notifyClients();
	}
    
    public String getMessage(int number) {
		return messages.get(number);
	}
    
    public int getMessagesSize() {
		return messages.size();
	}

}